/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Reseau.h"
#include "Declaration.h"


char      nbtname[] = {
	0x80, 0xf0, 0x00, 0x10, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x20, 0x43, 0x4b, 0x41, 0x41, 0x41, 0x41, 0x41,
	0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
	0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
	0x41, 0x41, 0x41, 0x41, 0x41, 0x00, 0x00, 0x21, 0x00, 0x01
};


gboolean Demande_NetBios (struct struct_Machines_Resultat *Resultat, int Machine, int Tentative)	// boolean
{
	int       Socket, i, octet_recu;
	struct sockaddr_in server;
	struct hostent *host;
	unsigned char buf[255];
	char     *FinChaine;
	char      Tmp[255];

	if ((host = gethostbyname (Liste_Machines[Machine].Adresse_Ip)) == NULL) {
		debug_printf(D_ERRO,"[S] (Demande_NetBios) Envoie demande NetBios : resolution erreur\n");
		return FALSE;	// boolean
	}
	if ((Socket = socket_control (PF_INET, SOCK_DGRAM, UDP)) < 0) {
		perror ("[S] (Demande_NetBios) Erreur socket");
		return FALSE;	// boolean
	}
	bzero ((char *) &server, sizeof (server));
	bcopy (host->h_addr, (char *) &server.sin_addr, host->h_length);
	server.sin_family = host->h_addrtype;
	server.sin_port = htons (NETBIOS);
	Nouveau_Octet_Envoye (sizeof (nbtname));
	if (sendto (Socket, nbtname, sizeof (nbtname), 0, (struct sockaddr *) &server, sizeof (server)) < 0) {
		sprintf (Tmp, "[S] Erreur (Demande_NetBios): Envoie packet (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
		perror (Tmp);
		if (close_control (Socket) < 0) {
			sprintf (Tmp, "[S] Erreur (Demande_NetBios): close (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
			perror (Tmp);
			debug_printf(D_ERRO,"[S] ERREUR (Demande_NetBios) Close (%d)   %d   (%s) Socket(%d) \n", Machine, errno, strerror (errno), Socket);
		}
		return FALSE;	// boolean
	}
	else {
		debug_printf(D_ALL,"[S][%d] Demande NETBIOS tentative(%d)\n", Machine, Tentative);
		if (waiting_donnee (Socket, 3000)) {
			octet_recu = read (Socket, buf, sizeof (buf) - 1);
			Nouveau_Octet_Recu (octet_recu);
			debug_printf(D_ALL,"[S][%d] Nombre d'enregistrement NetBios (%d)\n", Machine, buf[56]);
			if (buf[56] == 0) {
				if (close_control (Socket) < 0) {
					sprintf (Tmp, "[S] Erreur (Demande_NetBios): close (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
					perror (Tmp);
					debug_printf(D_ERRO,"[S] ERREUR (Demande_NetBios) Close (%d)   %d   (%s) Socket(%d) \n", Machine, errno, strerror (errno), Socket);
				}
				return FALSE;	// boolean
			}
			for (i = 0; i < buf[56]; i++) {
				strncpy (Resultat->Smb.Smb_Registration[i].Name, &buf[57 + (i * 18)], 15);
				Resultat->Smb.Smb_Registration[i].Name[15] = 0;
				/* On enleve les espaces finaux */
				if ((FinChaine = index (Resultat->Smb.Smb_Registration[i].Name, 0x20)) != NULL)
					FinChaine[0] = 0;
				/* recupere le numero de l'enregistrement */
				Resultat->Smb.Smb_Registration[i].Numero = buf[57 + 15 + (i * 18)];
				/* recupere le type d'enregistrement */
				if (buf[57 + 15 + 1 + (i * 18)] & 128) {
					Resultat->Smb.Smb_Registration[i].Type = REGISTRATION_GROUP;
					debug_printf(D_ALL,"[S][%d] GROUP %s <%02X>\n", Machine, Resultat->Smb.Smb_Registration[i].Name, Resultat->Smb.Smb_Registration[i].Numero);
				}
				else {
					Resultat->Smb.Smb_Registration[i].Type = REGISTRATION_UNIQUE;
					debug_printf(D_ALL,"[S][%d] UNIQUE   %s <%02X>\n", Machine, Resultat->Smb.Smb_Registration[i].Name, Resultat->Smb.Smb_Registration[i].Numero);
				}	/* recupere le nom de la machine */
				if ((Resultat->Smb.Smb_Registration[i].Numero == 0x20) && (Resultat->Smb.Smb_Registration[i].Type = REGISTRATION_UNIQUE))
					strcpy (Resultat->Smb.HostName, Resultat->Smb.Smb_Registration[i].Name);
				/* recupere le nom du domaine */
				if ((Resultat->Smb.Smb_Registration[i].Numero == 0x00) && (Resultat->Smb.Smb_Registration[i].Type == REGISTRATION_GROUP))
					strcpy (Resultat->Smb.DomainName, Resultat->Smb.Smb_Registration[i].Name);
				/* recupere le nom de l'utilisateur */
				if ((Resultat->Smb.Smb_Registration[i].Numero == 0x03) && (Resultat->Smb.Smb_Registration[i].Type == REGISTRATION_UNIQUE))
					strcpy (Resultat->Smb.Utilisateur, Resultat->Smb.Smb_Registration[i].Name);
			}
		}
		else {
			debug_printf(D_ALL,"[S][%d] Aucune reponse NETBIOS tentatice(%d)\n", Machine, Tentative);
			if (close_control (Socket) < 0) {
				sprintf (Tmp, "[S] Erreur (Demande_NetBios): close (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
				perror (Tmp);
				debug_printf(D_ERRO,"[S] ERREUR (Demande_NetBios) Close (%d)   %d   (%s) Socket(%d) \n", Machine, errno, strerror (errno), Socket);
			}
			return FALSE;	// boolean
		}
	}
	/* le nom netbios et le nom de domaine doivent etre absolument trouve sinon nouvelle tentative */
	if ((Resultat->Smb.HostName[0] != 0) && (Resultat->Smb.DomainName[0] != 0)) {
		if (close_control (Socket) < 0) {
			sprintf (Tmp, "[S] Erreur (Demande_NetBios): close (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
			perror (Tmp);
			debug_printf(D_ERRO,"[S] ERREUR (Demande_NetBios) Close (%d)   %d   (%s) Socket(%d) \n", Machine, errno, strerror (errno), Socket);
		}
		return TRUE;	// boolean
	}

	if (close_control (Socket) < 0) {
		sprintf (Tmp, "[S] Erreur (Demande_NetBios): close (%d)[%s]", Machine, Liste_Machines[Machine].Adresse_Ip);
		perror (Tmp);
		debug_printf(D_ERRO,"[S] ERREUR (Demande_NetBios) Close (%d)   %d   (%s) Socket(%d) \n", Machine, errno, strerror (errno), Socket);
	}
	return FALSE;		// boolean
}
