 /*
  *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <fcntl.h>
#include <asm/poll.h>

#include "Declaration.h"
#include "Reseau.h"

/*
*  attend un événement sur un descripteur
* ------------------------------------
*  sock : num de la socket
*  Gestion de erreurs : FALSE après le timeout de 300 secondes
*/
#define  DONNEE_PRET 1
#define  ATTENDRE    2
#define  ERREUR_SOCK 3		// erreur ou connection failed

int waiting_service (int sock, int event)
{
	struct pollfd fds;
	int       resultat;

	bzero (&fds, sizeof fds);	// on mets la structure a zero
	fds.events = event;
	fds.fd = sock;		// je scrute mon socket 
	if ((resultat = poll (&fds, 1, 1000)) > 0) {	// attend 1 seconde
		printf ("[%02X]", fds.revents);
		if ((fds.revents & POLLOUT) == POLLOUT) {
			printf ("Une écriture ne bloquera pas\n");
		}
		if ((fds.revents & POLLERR) == POLLERR) {
			printf ("Condition d'erreur\n");
			return ERREUR_SOCK;
		}
		if ((fds.revents & POLLHUP) == POLLHUP) {
			printf ("Déconnexion\n");
			return ERREUR_SOCK;
		}
		return DONNEE_PRET;
	}
	else {
		if (resultat != 0) {
			perror ("Erreur de poll");
			return ERREUR;
		}
		printf ("A[%02X]resultat(%d)", fds.revents, resultat);
		return ATTENDRE;
	}
}

/*
*  attend la donnée
* ------------------------------------
*  sock : num de la socket
*  Gestion de erreurs : FALSE après le timeout de 300 secondes
*/
gboolean attendre_Donnee (int sock, int temp)
{
	int       result;

	//i = 5;
	//printf("Attendre_donnee\n");
	while ((result = waiting_service (sock, POLLIN)) == ATTENDRE) {
		printf (".(%d)\n", temp);
		temp--;
		if (temp == 0) {
			printf ("Timeout dans Attendre_donnee\n");
			return FALSE;
		}
	}
	if (result == ERREUR_SOCK)
		return FALSE;
	return TRUE;
}


/*
*  attend la connection avec host
* ------------------------------------
*  sock : num de la socket
*  Gestion de erreurs : FALSE après le timeout de 300 secondes
*/
gboolean attendre_Sock (int sock)
{
	int       i;
	int       result;

	i = 5;
	//printf("attendre_Sock\n");
	while ((result = waiting_service (sock, POLLOUT)) == ATTENDRE) {
		printf (".(%d)\n", i);
		i--;
		if (i == 0) {
			printf ("Timeout dans Attendre_sock\n");
			return FALSE;
		}
	}
	if (result == ERREUR_SOCK)
		return FALSE;
	return TRUE;
}


int ConnectionServeur (char *Serveur, int Port)
{
	int       IdSocket, code_erreur;
	struct hostent *host;
	struct sockaddr_in SocketAddr;
	char      MessageDerreur[255];

	IdSocket = socket (AF_INET, SOCK_STREAM, 0);	// Mode Connecté
	if (IdSocket == -1) {
		perror ("Erreur Création socket TCP/IP");
		return -1;
	}
	set_NonBlock (IdSocket);
	host = gethostbyname (Serveur);	// recupere l'adresse IP
	if (host == NULL) {
		switch (h_errno) {
		    case HOST_NOT_FOUND:
			    sprintf (MessageDerreur, "L'hôte indiqué est inconnu. [%s]\n", Serveur);
			    break;
		    case NO_ADDRESS:
			    sprintf (MessageDerreur, "Le nom est valide mais ne possède pas d'adresse IP. [%s]\n", Serveur);
			    break;
		    case NO_RECOVERY:
			    sprintf (MessageDerreur, "Une erreur fatale du serveur de noms est apparue. [%s]\n", Serveur);
			    break;
		    case TRY_AGAIN:
			    sprintf (MessageDerreur, "Aucune connection à internet. [%s]\n", Serveur);
			    break;
		    default:
			    sprintf (MessageDerreur, "Erreur Inconnue [%s]\n", Serveur);
			    break;
		}
		close (IdSocket);
		return (-1);
	}
	memset (&SocketAddr, 0, sizeof (struct sockaddr_in));
	memcpy (&SocketAddr.sin_addr.s_addr, host->h_addr, host->h_length);
	SocketAddr.sin_family = AF_INET;
	SocketAddr.sin_port = htons (Port);
	if ((code_erreur = connect (IdSocket, (struct sockaddr *) &SocketAddr, sizeof (struct sockaddr))) < 0) {
		//printf("[S] warning Connection (%s) port %d   code_erreur(%d)\n",Serveur,Port,code_erreur);
		if (errno != EINPROGRESS) {
			perror ("Erreur de Connection");
			return -1;
		}
		if (attendre_Sock (IdSocket) == FALSE) {
			return -1;
		}
	}
	return IdSocket;
}






char     *Recupere_Banniere_FTP (char *serveur, int Machine)
{
#define   BANNIERE_FTP  "FTP : "
	int       IdSocket, NbOctet, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine, *banniere;
	FILE     *In;
	gboolean  fin;


	printf ("[S] [%d]--------------------- Recuperation FTP --------------------\n", Machine);
	if ((IdSocket = ConnectionServeur (serveur, 21)) == -1) {
		printf ("[S] [%d] (Recupere_Banniere_FTP) Connexion refusée au serveur FTP\n", Machine);
		return NULL;
	}
	else {
		In = fdopen (IdSocket, "r");
		if (In == NULL) {
			perror ("[S] (Recupere_Banniere_FTP) Erreur à l'ouverture du flux en lecture");
			return NULL;
		}
		strcpy (Requete, "Bad command\n");
		NbOctet = send (IdSocket, Requete, strlen (Requete), 0);
		if (NbOctet < 0) {
			return NULL;
		}
		Nouveau_Octet_Envoye (NbOctet);
		TailleBanniere = sizeof (BANNIERE_FTP);
		TailleChaine = 0;
		banniere = malloc (sizeof (char) * TailleBanniere);
		strcpy (banniere, BANNIERE_FTP);
		//do {
		//chaine = NULL;
		fin = FALSE;
		while ((!fin)&&(attendre_Donnee (IdSocket, 5))) {
			while ((chaine = fgets (Requete, sizeof (Requete), In)) != NULL) {
				//if (chaine != NULL) {
				Nouveau_Octet_Recu (strlen (chaine));
				printf ("[%s]\n", chaine);
				TailleChaine = strlen (chaine);
				banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
				strcat (banniere, chaine);
				TailleBanniere = TailleBanniere + TailleChaine;
				if (strncmp (chaine, "220", 3) != 0) {
					printf ("[S] [%d] (Recupere_Banniere_FTP) Fin des donnees a recuperer\n", Machine);
					fin = TRUE;
				}
				//}
				//else
				//printf ("[S] [%d] (Recupere_Banniere_FTP) Fin plus de donnees\n", Machine);
			}
		}
		/*}
		 * else {
		 * printf ("[S] [%d] (Recupere_Banniere_FTP) problème détecté\n", Machine);
		 * chaine = fgets (Requete, sizeof (Requete), In);
		 * printf ("%s\n", chaine);
		 * }
		 * } while (chaine != NULL); */
		//  test debugage
		if (TailleBanniere != (strlen (banniere) + 1))
			printf ("[S] [%d] (Recupere_Banniere_FTP) Erreur TailleBanniere     TailleBanniere[%d]   banniere[%d] \n", Machine, TailleBanniere, strlen (banniere));
		fclose (In);
		close (IdSocket);
		return banniere;
	}
}




char     *Recupere_Banniere_HTTP (char *serveur, int Machine)
{
#define   BANNIERE_HTTP  "HTTP : "
	int       IdSocket, NbOctet, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine, *banniere;
	FILE     *In;

	printf ("[S] [%d]--------------------- Recuperation HTTP --------------------\n", Machine);
	if ((IdSocket = ConnectionServeur (serveur, 80)) == -1) {
		printf ("[S] [%d] (Recupere_Banniere_HTTP) Connexion refusée au serveur HTTP\n", Machine);
		return NULL;
	}
	else {
		In = fdopen (IdSocket, "r");
		if (In == NULL) {
			printf ("[S] [%d] Recupere_Banniere_HTTP) Erreur à l'ouverture du flux en lecture\n", Machine);
			perror ("(Recupere_Banniere_HTTP) Erreur à l'ouverture du flux en lecture");
			return NULL;
		}
		sprintf (Requete, "GET / HTTP/1.0\r\nhost:%s\r\n\r\n\r\n", serveur);
		NbOctet = send (IdSocket, Requete, strlen (Requete), 0);
		if (NbOctet < 0) {
			return NULL;
		}
		Nouveau_Octet_Envoye (NbOctet);
		TailleBanniere = sizeof (BANNIERE_HTTP);
		TailleChaine = 0;
		banniere = malloc (sizeof (char) * TailleBanniere);
		strcpy (banniere, BANNIERE_HTTP);
		do {
			chaine = NULL;
			if (attendre_Donnee (IdSocket, 5)) {
				chaine = fgets (Requete, sizeof (Requete), In);
				if (chaine != NULL) {
					Nouveau_Octet_Recu (strlen (chaine));
					if (strncmp (chaine, "\r\n", 2) != 0) {
						TailleChaine = strlen (chaine);
						banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
						strcat (banniere, chaine);
						TailleBanniere = TailleBanniere + TailleChaine;
					}
					else {
						printf ("[S] [%d] (Recupere_Banniere_HTTP) Fin des donnees a recuperer\n", Machine);
						chaine = NULL;
					}
				}
				else
					printf ("[S] [%d] (Recupere_Banniere_HTTP) Fin plus de donnees\n", Machine);
			}
			else {
				printf ("[S] [%d] (Recupere_Banniere_HTTP) problème détecté\n", Machine);
			}
		} while (chaine != NULL);
		//  test debugage
		if (TailleBanniere != (strlen (banniere) + 1))
			printf ("[S] [%d] (Recupere_Banniere_HTTP) Erreur TailleBanniere     TailleBanniere[%d]   banniere[%d] \n", Machine, TailleBanniere, strlen (banniere));
		fclose (In);
		close (IdSocket);
		return banniere;
	}
}


char     *Recupere_Banniere_IMAP (char *serveur, int Machine)
{
#define   BANNIERE_IMAP  "IMAP : "
	int       IdSocket, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine, *banniere;
	FILE     *In;

	// Recuperation IMAP 
	printf ("[S] [%d]--------------------- Recuperation IMAP --------------------\n", Machine);
	if ((IdSocket = ConnectionServeur (serveur, 143)) == -1) {
		printf ("[S] [%d] (Recupere_Banniere_IMAP) Connexion refusée au serveur IMAP\n", Machine);
		return NULL;
	}
	else {
		In = fdopen (IdSocket, "r");
		if (In == NULL) {
			printf ("[S] [%d] Recupere_Banniere_IMAP) Erreur à l'ouverture du flux en lecture\n", Machine);
			perror ("(Recupere_Banniere_IMAP) Erreur à l'ouverture du flux en lecture");
			return NULL;
		}
		TailleBanniere = sizeof (BANNIERE_IMAP);
		TailleChaine = 0;
		banniere = malloc (sizeof (char) * TailleBanniere);
		strcpy (banniere, BANNIERE_IMAP);
		do {
			chaine = NULL;
			if (attendre_Donnee (IdSocket, 5)) {
				chaine = fgets (Requete, sizeof (Requete), In);
				if (chaine != NULL) {
					Nouveau_Octet_Recu (strlen (chaine));
					//printf ("%s", chaine);
					TailleChaine = strlen (chaine);
					banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
					strcat (banniere, chaine);
					TailleBanniere = TailleBanniere + TailleChaine;
					if (strstr (chaine, "\r\n") != NULL)
						chaine = NULL;
				}
				else
					printf ("[S] [%d] (Recupere_Banniere_IMAP) Fin plus de donnees\n", Machine);
			}
			else {
				printf ("[S] [%d] (Recupere_Banniere_IMAP) problème détecté\n", Machine);
			}
		} while (chaine != NULL);
		//  test debugage
		if (TailleBanniere != (strlen (banniere) + 1))
			printf ("[S] [%d] (Recupere_Banniere_IMAP) Erreur TailleBanniere     TailleBanniere[%d]   banniere[%d] \n", Machine, TailleBanniere, strlen (banniere));
		fclose (In);
		close (IdSocket);
		return banniere;
	}
}


char     *Recupere_Banniere_TELNET (char *serveur, int Machine)
{
#define   BANNIERE_TELNET  "TELNET : "
	int       IdSocket, NbOctet, Telnet_Recuperer, i, TailleBanniere;
	char      DonneeBrut[2000], *banniere;


	Telnet_Recuperer = -1;
	printf ("[S] [%d]--------------------- Recuperation TELNET --------------------\n", Machine);
	if ((IdSocket = ConnectionServeur (serveur, 23)) == -1) {
		printf ("[S] [%d] (Recupere_Banniere_TELNET) Connexion refusée au serveur TELNET\n", Machine);
		return NULL;
	}
	else {
		banniere = malloc (sizeof (char) * 10);
		strcpy (banniere, "TELNET : ");

		TailleBanniere = sizeof (BANNIERE_TELNET);
		banniere = malloc (sizeof (char) * TailleBanniere);
		strcpy (banniere, BANNIERE_TELNET);


		while ((Telnet_Recuperer == -1) && ((attendre_Donnee (IdSocket, 5) != FALSE) && ((NbOctet = recvfrom (IdSocket, &DonneeBrut, (sizeof (DonneeBrut) - 1), 0, NULL, 0)) > 0))) {	// -1 pour garder un caractere pour le fin de chaine 
			Nouveau_Octet_Recu (NbOctet);
			printf ("[S] [%d] TELNET %d octet\n", Machine, NbOctet);
			if ((DonneeBrut[0] & 0xff) == 0xff) {	// pas de donnee texte mais juste de la negociation du protocole telnet
				for (i = 0; i < (NbOctet / 3); i++) {
					DonneeBrut[(i * 3) + 1] = 0xfc;
				}
				NbOctet = send (IdSocket, DonneeBrut, NbOctet, 0);
				Nouveau_Octet_Envoye (NbOctet);
			}
			else {
				DonneeBrut[NbOctet] = 0;

				printf ("[S] [%d] TELNET :[%s]", Machine, DonneeBrut);
				banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + NbOctet + 1)));
				strcat (banniere, DonneeBrut);
				TailleBanniere = TailleBanniere + NbOctet;
				//  test debugage
				if (TailleBanniere != (strlen (banniere) + 1))
					printf ("[S] [%d] TELNET erreur TailleBanniere[%d]  strlen(banniere)[%d]\n", Machine, TailleBanniere, strlen (banniere));
				//Telnet_Recuperer = 1;
			}
		}
		close (IdSocket);
		return banniere;
	}
}
