/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <signal.h>

#include "Reseau.h"
#include "Sous_Reseau.h"
#include "Commun.h"
#include "Port.h"


static const char version[] = "AutoScan_d 0.95: compilé le " __TIME__ " " __DATE__ "\n";



void segfault_handler (int sig)
{
	debug_printf(D_ERRO,"\n[S] Segmentation fault    Et oui. Encore un bug   pid : %d Socket ouvert[%d]  Thread[%d]\n", getpid (), Nb_Socket_Ouverte, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan));
	Message ("", AUTOSCAN_SEGMENTATION_FAULT);
	Message ("autoscan_d : Segmentation fault", ERREUR);
	exit (0);
}

void Exit_Probleme_Trouver ()
{
	debug_printf(D_ERRO,"\n[S] Erreur    Et oui. Encore un bug   pid : %d    Socket ouvert[%d]  Thread[%d]\n", getpid (), Nb_Socket_Ouverte, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan));
	Message ("", AUTOSCAN_SEGMENTATION_FAULT);
	Message ("autoscan_d : Error", ERREUR);
	exit (0);
}

void Control_C (int sig)
{
	debug_printf(D_INFO,"[S] Demande d'arret  Socket ouvert[%d]  Thread[%d]\n",Nb_Socket_Ouverte, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan));
	debug_printf(D_INFO,"[S] Fin\n");
	Message ("", AUTOSCAN_D_ARRETER);
	exit (0);
}


void     *Boucle_Scan ()
{
	int       Machine;

	debug_printf(D_ALL,"[S] Demarrage boucle pid(%d)  scan    En cours de scan (%d) / (%d)\n", getpid (), Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan), Valeur_Semaphore (&Semaphore_Nouvelle_Machine));
	Machine = Nb_Machine_Dans_La_Liste;	// different de 0 si il y eu une restauration
	while (1) {
		sem_wait (&Semaphore_Nouvelle_Machine);
		sem_wait (&Semaphore_Thread_Scan);
		Message_Info ();
		debug_printf(D_ALL,"[S] Boucle scan n %d Ip = [%s]  Mac = [%s] En cours de scan (%d) / (%d)\n", Machine, Liste_Machines[Machine].Adresse_Ip, Liste_Machines[Machine].Adresse_Mac, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan), Valeur_Semaphore (&Semaphore_Nouvelle_Machine));
		demmarage_Thread (ScanMachine, Machine);
		Machine++;
	}
}



void     *Compteur_Utilisation_Reseau ()
{
	int       Envoye;
	int       Recu;
	struct struct_Message_Temps_Reel Message_Temps_Reel;

	debug_printf(D_ALL,"[S] Demarrage Compteur Utilisation reseau pid(%d)\n", getpid ());
	while (1) {
		sleep (1);
		sem_wait (&Semaphore_Calcul_Utilisation_Reseau);
		Envoye = Nb_Octet_Envoye;
		Recu = Nb_Octet_Recu;
		Nb_Octet_Envoye = 0;
		Nb_Octet_Recu = 0;
		sem_post (&Semaphore_Calcul_Utilisation_Reseau);
		Message_Temps_Reel.type = 1;
		Message_Temps_Reel.type_message = MISE_A_JOUR_UTILISATION_RESEAU;
		Message_Temps_Reel.Octet_Envoyer = Envoye;
		Message_Temps_Reel.Octet_Recu = Recu;
		if (msgsnd (File_Envoie_Message_Temps_Reel, (void *) &Message_Temps_Reel, sizeof (Message_Temps_Reel) - sizeof (long), 0) < 0) {
			perror ("[S] (Refresh) msgsnd erreur");
			Exit_Probleme_Trouver ();
		}
	}
}





void     *Boucle_demande ()
{
	struct struct_Demande Demande;
	int       Taille_Recuperer, i;
	unsigned long int AdresseIp;
	int       NumSousReseau;
	char      tmp[50], *pDonnee;
	struct sockaddr sock;
	int       Socket;


	debug_printf(D_ALL,"[S] Demarrage boucle demande\n");
	while (1) {
		if ((Taille_Recuperer = msgrcv (File_Envoie_Demande, (void *) &Demande, sizeof (Demande) - sizeof (long), 0, 0)) > -1) {
			switch (Demande.type_demande) {
			    case RE_SCAN_MACHINE:
				    debug_printf(D_ALL,"[S] Demande rescanne machine : %s\n", Demande.donnee);
				    i = 0;
				    while ((i < Nb_Machine_Dans_La_Liste) && (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) != 0)) {
					    i++;
				    }
				    if (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) == 0) {
					    debug_printf(D_ALL,"[S] Rescanne la machine : %s    Machine n%d\n", Demande.donnee, i);
					    Liste_Machines[i].Machine_ReScanner = TRUE;
					    demmarage_Thread (ScanMachine, i);
				    }
				    else {
					    sprintf (tmp, "unreachable machine : %s\n", Demande.donnee);
					    Message (tmp, ERREUR);

				    }
				    break;
			    case SCAN_SOUS_RESEAU:
				    debug_printf(D_ALL,"[S] Rescanne un sous reseau : %s\n", Demande.donnee);
				    AdresseIp = inet_addr (Demande.donnee);
				    if (AdresseIp != INADDR_NONE) {
					    /* Stocke les nouveaux sous rseau */
					    NumSousReseau = 0;
					    while ((Sous_Reseau[NumSousReseau].Reseau[0] != 0)
						    && (memcmp (&Sous_Reseau[NumSousReseau].Reseau, &AdresseIp, IP_ADDR_LEN - 1) != 0)) {
						    NumSousReseau++;
					    }
					    if (Sous_Reseau[NumSousReseau].Reseau[0] == 0) {
						    memcpy (&Sous_Reseau[NumSousReseau].Reseau, &AdresseIp, IP_ADDR_LEN - 1);
						    demmarage_Thread (Nouveau_Sous_Reseau, NumSousReseau);
					    }
					    else {
						    debug_printf(D_ALL,"[S] Ce sous reseau existe deja\n");
						    Message ("Subnet is already existing", INFO);
					    }
				    }
				    else
					    debug_printf(D_ERRO,"[S] Adresse Incorrecte\n");
				    break;
			    case MACHINE_RESTAURER:
				    debug_printf(D_ALL,"[S] Machine restaurer : (%s)    [%s] \n", Demande.Adresse_Ip, Demande.Adresse_Mac);
				    strcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Ip, Demande.Adresse_Ip);
				    strcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Mac, Demande.Adresse_Mac);
				    Liste_Machines[Nb_Machine_Dans_La_Liste].AdresseIp = Demande.AdresseIp;
				    memcpy (Liste_Machines[Nb_Machine_Dans_La_Liste].AdresseMAC, Demande.AdresseMAC, 6);
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Scan_Port_En_Cours = FALSE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_Restaurer = TRUE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_ReScanner = FALSE;
				    Liste_Machines[Nb_Machine_Dans_La_Liste].Machine_Allumer = FALSE;	// Cette machine est restaurer donc pas dfaut on considre qu'elle est teint
				    debug_printf(D_ALL,"[S] Nouvelle Machine (%d) : (%s)  [%s]  \n", Nb_Machine_Dans_La_Liste, Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Ip, Liste_Machines[Nb_Machine_Dans_La_Liste].Adresse_Mac);
				    Nb_Machine_Dans_La_Liste++;
				    break;
			    case DEMANDE_ARRET_AUTOSCAN_D:
				    debug_printf(D_INFO,"[S] Demande d'arret recu Socket ouvert[%d]  Thread[%d]\n", Nb_Socket_Ouverte, Nb_Max_Thread - Valeur_Semaphore (&Semaphore_Thread_Scan));
				    debug_printf(D_INFO,"[S] Fin\n");
				    Message ("", AUTOSCAN_D_ARRETER);
				    exit (0);
				    break;
			    case DEMARRAGE_SANS_SCAN:
				    debug_printf(D_ALL,"[S] Interdiction de scan\n");
				    Message_Info ();
				    debug_printf(D_ALL,"[S] Interface Reseau selectionner : [%s]    Max thread : (%d)\n", Demande.interface, Demande.Nb_Thread_Max);
				    /* Interface au demarrage */
				    Interface = (char *) malloc (strlen (Demande.interface) + 1);
				    strcpy (Interface, Demande.interface);
				    debug_printf(D_ALL,"[S] Donnee [%s]\n", Demande.donnee);
				    pDonnee = index (Demande.donnee, ';');
				    strncpy (pwd_Utilisateur, Demande.donnee, pDonnee - Demande.donnee);
				    pDonnee++;
				    strncpy (Nessus_Login, pDonnee, index (pDonnee, ';') - pDonnee);
				    pDonnee = index (pDonnee, ';') + 1;
				    strncpy (Nessus_Password, pDonnee, index (pDonnee, ';') - pDonnee);
				    strcat (Nessus_Login, "\n");	// obligatoire pour le client NEssus
				    strcat (Nessus_Password, "\n");
				    Nb_Max_Thread = Demande.Nb_Thread_Max;
				    sem_init (&Semaphore_Nouvelle_Machine, 0, 0);
				    sem_init (&Semaphore_Thread_Scan, 0, Nb_Max_Thread);
				    Autoriser_Scan = FALSE;
				    strcpy (sock.sa_data, Interface);
				    sock.sa_family = AF_INET;
				    if ((Socket = socket_control (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
					    perror ("[S] Erreur Socket: ");
					    Exit_Probleme_Trouver ();
				    }
				    debug_printf(D_ALL,"[S] ********* Information ***********\n");
				    Get_My_MAC (Socket, Interface);
				    Get_My_Ip (Socket, Interface);
				    debug_printf(D_ALL,"[S]  *  pwd de l'utilisateur : [%s]\n", pwd_Utilisateur);
				    debug_printf(D_ALL,"[S]  *  Nessus Login : [%s]\n", Nessus_Login);
				    debug_printf(D_ALL,"[S]  *  Nessus Password : [%s]\n", Nessus_Password);
				    debug_printf(D_ALL,"[S] *********************************\n");
				    close_control (Socket);
				    Creer_PortList ();
				    Creer_SignatureList ();
				    break;
			    case DEMARRAGE_SCAN:
				    debug_printf(D_ALL,"[S] Autorisation de scan accord\n");
				    Message_Info ();
				    debug_printf(D_ALL,"[S] Interface Reseau selectionner : [%s]    Max thread : (%d)\n", Demande.interface, Demande.Nb_Thread_Max);
				    /* Interface au demarrage */
				    Interface = (char *) malloc (strlen (Demande.interface) + 1);
				    strcpy (Interface, Demande.interface);
				    debug_printf(D_ALL,"[S] Donnee [%s]\n", Demande.donnee);
				    pDonnee = index (Demande.donnee, ';');
				    strncpy (pwd_Utilisateur, Demande.donnee, pDonnee - Demande.donnee);
				    pDonnee++;
				    strncpy (Nessus_Login, pDonnee, index (pDonnee, ';') - pDonnee);
				    pDonnee = index (pDonnee, ';') + 1;
				    strncpy (Nessus_Password, pDonnee, index (pDonnee, ';') - pDonnee);
				    strcat (Nessus_Login, "\n");	// obligatoire pour le client NEssus
				    strcat (Nessus_Password, "\n");
				    Nb_Max_Thread = Demande.Nb_Thread_Max;
				    sem_init (&Semaphore_Nouvelle_Machine, 0, 0);
				    sem_init (&Semaphore_Thread_Scan, 0, Nb_Max_Thread);
				    strcpy (sock.sa_data, Interface);
				    sock.sa_family = AF_INET;
				    if ((Socket = socket_control (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0) {
					    perror ("[S] Erreur Socket: ");
					    Exit_Probleme_Trouver ();
				    }
				    debug_printf(D_ALL,"[S] ********* Information ***********\n");
				    Get_My_MAC (Socket, Interface);
				    Get_My_Ip (Socket, Interface);
				    debug_printf(D_ALL,"[S]  *  pwd de l'utilisateur : [%s]\n", pwd_Utilisateur);
				    debug_printf(D_ALL,"[S]  *  Nessus Login : [%s]\n", Nessus_Login);
				    debug_printf(D_ALL,"[S]  *  Nessus Password : [%s]\n", Nessus_Password);
				    debug_printf(D_ALL,"[S] *********************************\n");
				    close_control (Socket);
				    Creer_PortList ();
				    Creer_SignatureList ();
				    Creer_Liste_Sous_Reseau ();
				    i = 0;
				    while (Sous_Reseau[i].Reseau[0] != 0) {
					    if (!Sous_Reseau[i].Deja_Scanner) {	// probleme rescan 
						    demmarage_Thread (Nouveau_Sous_Reseau, i);
					    }
					    i++;
				    }
				    Autoriser_Scan = TRUE;
				    demmarage_Thread (Thread_Recherche_Ip, 0);
				    demmarage_Thread (Boucle_Scan, 0);
				    demmarage_Thread (Compteur_Utilisation_Reseau, 0);
				    break;
			    case ARRET_SCAN:
				    Autoriser_Scan = FALSE;
				    break;
			    case RELECTURE_FICHIER_CONFIGURATION:
				    Relecture_Fichiers_Configuration = TRUE;
				    break;
			    case REFRESH_MACHINE:
				    debug_printf(D_ALL,"[S] Demande refresh machine : %s\n", Demande.donnee);
				    i = 0;
				    while ((i < Nb_Machine_Dans_La_Liste) && (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) != 0)) {
					    i++;
				    }
				    if (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) == 0) {
					    debug_printf(D_ALL,"[S] Refresh la machine : %s    Machine n%d\n", Demande.donnee, i);
					    Refresh (i);
				    }
				    else {
					    sprintf (tmp, "unreachable machine : %s\n", Demande.donnee);
					    Message (tmp, ERREUR);
				    }
				    break;
			    case SCANNE_VULNERABILITE:
				    debug_printf(D_ALL,"[S] Scanne les vulnerabilites\n");
				    i = 0;
				    while ((i < Nb_Machine_Dans_La_Liste) && (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) != 0)) {
					    i++;
				    }
				    if (strcmp (Liste_Machines[i].Adresse_Ip, Demande.donnee) == 0) {
					    debug_printf(D_ALL,"[S] Scanne vulnerabilites machine : %s    Machine n%d\n", Demande.donnee, i);
					    demmarage_Thread (Client_Nessus, i);
				    }
				    break;
			    default:
				    debug_printf(D_ERRO,"[S] Type de la demande Inconnu\n");
			}
		}
		else {
			debug_printf(D_ERRO,"[S] Erreur taille de demanden");
			Exit_Probleme_Trouver ();
		}
	}
}



int main (int argc, char *argv[])
{
	Debug = D_INFO;
	Nb_Machine_Dans_La_Liste = 0;
	Autoriser_Scan = FALSE;
	Relecture_Fichiers_Configuration = FALSE;
	debug_printf(D_INFO,version);
	Nb_Octet_Envoye = 0;
	Nb_Octet_Recu = 0;
	/* Elevation des privileges */
	setuid (0);
	seteuid (0);
	if ((getuid () != 0) || (geteuid () != 0)) {
		debug_printf(D_ERRO,"[S] Root Only\n");
		exit (0);
	}
	if ((File_Reception_Nouvelle_Machine = msgget (KEY_FILE_RECEPTION_NOUVELLE_MACHINE, 0)) == -1) {
		perror ("[S] msgget");
		debug_printf(D_ERRO,"Lancer 'AutoScan'\n");
		exit (1);
	}
	if ((File_Envoie_Demande = msgget (KEY_FILE_DEMANDE, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	if ((File_Envoie_Message = msgget (KEY_FILE_MESSAGE, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	if ((File_Envoie_Message_Temps_Reel = msgget (KEY_FILE_MESSAGE_TEMP_REEL, 0)) == -1) {
		perror ("[S] msgget");
		exit (1);
	}
	debug_printf(D_ALL,"[S] Attachement des files des messages : OK\n");
	debug_printf(D_ALL,"Taille struct_Machines_Resultat : %d\n", sizeof (struct struct_Machines_Resultat));
	signal (SIGSEGV, segfault_handler);
	signal (SIGINT, Control_C);
	sem_init (&Semaphore_Calcul_Utilisation_Reseau, 0, 1);
	sem_init (&Semaphore_Control_Usage_Socket, 0, 1);
	Nb_Socket_Ouverte = 0;

	Initialise_Structure_Sous_Reseau ();
	demmarage_Thread (Boucle_demande, 0);

	while (1) {
		pause ();
	}
	return 0;
}
