 /*
  *  Copyright (C) 2005 Thierry Lagarde <tlagarde@user.sourceforge.net>
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */


#include "Gconf.h"
#include "Reseau.h"
#include "callbacks.h"
#include "Debug.h"


gboolean gconf_error (GError ** err)
{
	if (*err != NULL) {
		debug_printf (D_ERRO, "[I]  gconf_error : [%s]\n", (*err)->message);
		g_error_free (*err);
		*err = NULL;
		return TRUE;
	}
	return FALSE;
}

char     *Lecture_Info_Conf_Char (char *path)
{
	char     *Donnee;
	GError   *err = NULL;

	Donnee = gconf_client_get_string (client_gconf, path, &err);
	if (!gconf_error (&err) && (Donnee != NULL)) {
		debug_printf (D_ALL, "%s \n", Donnee);
		return Donnee;
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Lecture_Info_Conf_Char) : [%s]\n", path);
		return NULL;
	}
}


gboolean Lecture_Info_Conf_Bool (char *path)
{
	gboolean  Donnee;
	GError   *err = NULL;

	Donnee = gconf_client_get_bool (client_gconf, path, &err);
	if (!gconf_error (&err)) {
		debug_printf (D_ALL, "%d \n", Donnee);
		return Donnee;
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Lecture_Info_Conf_Bool) : [%s]\n", path);
		return FALSE;
	}
}


int Lecture_Info_Conf_Int (char *path)
{
	int       Donnee;
	GError   *err = NULL;

	Donnee = gconf_client_get_int (client_gconf, path, &err);
	if (!gconf_error (&err)) {
		debug_printf (D_ALL, "%d \n", Donnee);
		return Donnee;
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Lecture_Info_Conf_Int) : [%s]\n", path);
		return 0;
	}
}







void Enregistrement_Info_Conf_Char (char *path, char *Donnee)
{
	GError   *err = NULL;
	gboolean  retour;

	retour = gconf_client_set_string (client_gconf, path, Donnee, &err);
	if (!retour || !gconf_error (&err)) {
		debug_printf (D_ALL, "%s \n", Donnee);
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Enregistrement_Info_Conf_Char) : [%s]\n", path);
	}
}


void Enregistrement_Info_Conf_Bool (char *path, gboolean Donnee)
{
	GError   *err = NULL;
	gboolean  retour;

	retour = gconf_client_set_bool (client_gconf, path, Donnee, &err);
	if (!retour || !gconf_error (&err)) {
		debug_printf (D_ALL, "%d \n", Donnee);
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Enregistrement_Info_Conf_Bool) : [%s]\n", path);
	}
}


void Enregistrement_Info_Conf_Int (char *path, int Donnee)
{
	GError   *err = NULL;
	gboolean  retour;

	retour = gconf_client_set_int (client_gconf, path, Donnee, &err);
	if (!retour || !gconf_error (&err)) {
		debug_printf (D_ALL, "%d \n", Donnee);
	}
	else {
		debug_printf (D_ERRO, "[I] Erreur (Enregistrement_Info_Conf_Int) : [%s]\n", path);
	}
}



void Affiche_Champ_Connection_Internet ()
{
	char      Num_Port[10];

	if (strcmp (Lecture_Info_Conf_Char ("/system/proxy/mode"), "manual") == 0) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_Connection_Internet_Manuel), TRUE);
	}
	if (strcmp (Lecture_Info_Conf_Char ("/system/proxy/mode"), "auto") == 0) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_Connection_Internet_Auto), TRUE);
	}
	sprintf (Num_Port, "%d", Lecture_Info_Conf_Int ("/system/http_proxy/port"));
	gtk_entry_set_text (GTK_ENTRY (entry_Connection_Internet_Manuel_Port), Num_Port);
	gtk_entry_set_text (GTK_ENTRY (entry_Connection_Internet_Manuel_Proxy), Lecture_Info_Conf_Char ("/system/http_proxy/host"));
	gtk_entry_set_text (GTK_ENTRY (entry_Connection_Internet_Auto), Lecture_Info_Conf_Char ("/system/proxy/autoconfig_url"));
}
